<?php

class ConceptoDAO extends CI_Model 
{
    public function __construct()
    {    
        parent::__construct();
    }

    public function getConcepto($oConceptoBE, &$retorno, &$mensaje)
    {
        $idConcepto = $oConceptoBE->idConcepto;
        $idTipo = $oConceptoBE->idTipo;

        $listResult = array();
              
        $sql = "SELECT
                    T1.idconcepto,
                    T1.idtipo,
                    DATE_FORMAT(T1.fecha, '%d/%m/%Y') AS fecha,
                    T1.empresa,
                    T1.descripcion,
                    T1.responsable,
                    T1.monto,	
                    (SELECT TT.descripcion FROM parametro TT WHERE TT.tipparam = 'TIPO_CONCEPTO' AND TT.codigo = T1.idtipo AND TT.eliminado = 0 LIMIT 1) AS dsctipo
                FROM
                    concepto T1
                WHERE
                    COALESCE(?,0) IN (T1.idconcepto,0)
                    AND COALESCE(?,'') IN (T1.idtipo,'')
                    AND T1.eliminado = 0
                ORDER BY
                    3";
        
        $query = $this->db->query($sql, array($idConcepto, $idTipo));

        if($query)
        {
          $list = $query->result();

          foreach ($list as $row)
          {
            $oConceptoBE = new ConceptoBE();

            $oConceptoBE->idConcepto = (int)$row->idconcepto;
            $oConceptoBE->idTipo = $row->idtipo;
            $oConceptoBE->fecha = $row->fecha;
            $oConceptoBE->empresa = $row->empresa;
            $oConceptoBE->descripcion = $row->descripcion;
            $oConceptoBE->responsable = $row->responsable;
            $oConceptoBE->monto = (double)$row->monto;

            array_push($listResult, $oConceptoBE);
          }
          
          $retorno = 0;
          $mensaje = MSG_OK_GENERAL;
        }
        else
        {
          $retorno = 1;
          $mensaje = MSG_ERROR_GENERAL;
        }

        return $listResult;
    }
    
    public function createConcepto($oConceptoBE, &$retorno, &$mensaje)
    {
        $this->load->model('UtilDAO', 'util');

        $objResult = null;
                        
        $idTipo = $oConceptoBE->idTipo;
        $fecha = $oConceptoBE->fecha;
        $empresa = $oConceptoBE->empresa;
        $descripcion = $oConceptoBE->descripcion;
        $responsable = $oConceptoBE->responsable;
        $monto = $oConceptoBE->monto;
        $idUsuCrea = $oConceptoBE->idUsuCrea;

        $idConcepto = $this->util->getNextId("concepto","idconcepto");
                
        if ($idConcepto > 0)
        {
            $sql = "INSERT INTO concepto
                    (
                        idconcepto,
                        idtipo,
                        fecha,
                        empresa,
                        descripcion,
                        responsable,
                        monto,
                        idusucrea,
                        fecusucrea,
                        idusumod,
                        fecusumod,
                        eliminado
                    )
                    VALUES
                    (
                        ?,	
                        ?,
                        ?,
                        ?,
                        ?,
                        ?,
                        ?,	
                        ?,
                        NOW(),
                        NULL,
                        NULL,
                        0
                    )";
                
            $query = $this->db->query($sql, array($idConcepto,
                                                  $idTipo,
                                                  $fecha,
                                                  $empresa,
                                                  $descripcion,
                                                  $responsable,
                                                  $monto,
                                                  $idUsuCrea));

            if($query)
            {
                $oConceptoBE = new ConceptoBE();

                $oConceptoBE->idConcepto = $idConcepto;

                $listResult = $this->getConcepto($oConceptoBE, $retorno, $mensaje);

                if(count($listResult) > 0)
                {
                    $objResult = $listResult[0];

                    $retorno = 0;
                    $mensaje = "Concepto grabado correctamente";
                }
                else
                {
                    $retorno = 1;
                    $mensaje = MSG_ERROR_GENERAL;
                }
            }
            else
            {
                $retorno = 1;
                $mensaje = MSG_ERROR_GENERAL;
            }         
        }
        else
        {
            $retorno = 1;
            $mensaje = "Ocurrió un error al obtener el codigo de Concepto";
        }

        return $objResult;
    }

    public function updateConcepto($oConceptoBE, &$retorno, &$mensaje)
    {
        $this->load->model('UtilDAO', 'util');

        $objResult = null;                

        $idConcepto = $oConceptoBE->idConcepto;
        $idTipo = $oConceptoBE->idTipo;
        $fecha = $oConceptoBE->fecha;
        $empresa = $oConceptoBE->empresa;
        $descripcion = $oConceptoBE->descripcion;
        $responsable = $oConceptoBE->responsable;
        $monto = $oConceptoBE->monto;
        $idUsuMod = $oConceptoBE->idUsuMod;
        
        if ($idConcepto > 0)
        {
            $oConceptoBE = new ConceptoBE();

            $oConceptoBE->idConcepto = $idConcepto;

            $listResult = $this->getConcepto($oConceptoBE, $retorno, $mensaje);

            if(count($listResult) > 0)
            {
                $sql = "UPDATE 
                            concepto
                        SET
                            idtipo = COALESCE(?,idtipo),
                            fecha = COALESCE(?,fecha),
                            empresa = COALESCE(?,empresa),
                            descripcion = COALESCE(?,descripcion),
                            responsable = COALESCE(?,responsable),
                            monto = COALESCE(?,monto),
                            idusumod = COALESCE(?,idusumod),
                            fecusumod = NOW()
                        WHERE
                            idconcepto = ?
                            AND eliminado = 0";
                    
                $query = $this->db->query($sql, array($idTipo,
                                                      $fecha,
                                                      $empresa,
                                                      $descripcion,
                                                      $responsable,
                                                      $monto,                                                      
                                                      $idUsuMod,
                                                      $idConcepto));

                if($query)
                {
                    $oConceptoBE = new ConceptoBE();

                    $oConceptoBE->idConcepto = $idConcepto;

                    $listResult = $this->getConcepto($oConceptoBE, $retorno, $mensaje);

                    if(count($listResult) > 0)
                    {
                        $objResult = $listResult[0];

                        $retorno = 0;
                        $mensaje = "Concepto grabado correctamente";
                    }
                    else
                    {
                        $retorno = 1;
                        $mensaje = MSG_ERROR_GENERAL;
                    }
                }
                else
                {
                    $retorno = 1;
                    $mensaje = MSG_ERROR_GENERAL;
                }
            }
            else
            {
                $retorno = 1;
                $mensaje = "El Concepto no existe";
            }
        }
        else
        {
            $retorno = 1;
            $mensaje = "Debe ingresar el codigo de Concepto";
        }

        return $objResult;
    }

    public function deleteConcepto($oConceptoBE, &$retorno, &$mensaje)
    {
        $this->load->model('UtilDAO', 'util');

        $objResult = null;                

        $idConcepto = $oConceptoBE->idConcepto;        
        $idUsuMod = $oConceptoBE->idUsuMod;
        
        if ($idConcepto > 0)
        {
            $oConceptoBE = new ConceptoBE();

            $oConceptoBE->idConcepto = $idConcepto;

            $listResult = $this->getConcepto($oConceptoBE, $retorno, $mensaje);

            if(count($listResult) > 0)
            {                
                $sql = "UPDATE 
                           concepto
                        SET
                           idusumod = ?,
                           fecusumod = NOW(),
                           eliminado = 1
                        WHERE
                           idconcepto = ?
                           AND eliminado = 0";
                    
                $query = $this->db->query($sql, array($idUsuMod,
                                                      $idConcepto));
                if($query)
                {
                    $retorno = 0;
                    $mensaje = "Concepto eliminado correctamente";
                }
                else
                {
                    $retorno = 1;
                    $mensaje = MSG_ERROR_GENERAL;
                }
            }
            else
            {
                $retorno = 1;
                $mensaje = "El Concepto no existe";
            }                    
        }
        else
        {
            $retorno = 1;
            $mensaje = "Debe ingresar el codigo de Concepto";
        }

        return $objResult;
    }

    public function getConceptoResumen($oConceptoBE, &$retorno, &$mensaje)
    {        
        $listResult = array();
              
        $sql = "SELECT 
                    idtipo, 
                    enero, 
                    febrero, 
                    marzo, 
                    abril, 
                    mayo, 
                    junio, 
                    julio, 
                    agosto, 
                    setiembre, 
                    octubre, 
                    noviembre, 
                    diciembre, 
                    (enero + febrero + marzo + abril + mayo + junio + julio + agosto + setiembre + octubre + noviembre + diciembre) AS total 
                FROM(
                SELECT	
                    '00001' AS idtipo,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 1 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS enero,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 2 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS febrero,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 3 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS marzo,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 4 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS abril,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 5 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS mayo,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 6 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS junio,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 7 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS julio,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 8 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS agosto,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 9 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS setiembre,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 10 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS octubre,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 11 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS noviembre,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 12 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00001'),0) AS diciembre
                UNION	
                SELECT	
                    '00002' AS idtipo,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 1 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS enero,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 2 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS febrero,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 3 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS marzo,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 4 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS abril,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 5 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS mayo,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 6 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS junio,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 7 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS julio,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 8 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS agosto,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 9 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS setiembre,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 10 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS octubre,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 11 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS noviembre,
                    COALESCE((SELECT SUM(TT.monto) FROM concepto TT WHERE MONTH(TT.fecha) = 12 AND YEAR(TT.fecha) = YEAR(NOW()) AND TT.idtipo = '00002'),0) AS diciembre	
                ) TR
                ORDER BY 
                    1";
        
        $query = $this->db->query($sql, array());

        if($query)
        {
          $list = $query->result();

          foreach ($list as $row)
          {
            $oConceptoBE = new ConceptoBE();

            $oConceptoBE->idtipo = $row->idtipo;
            $oConceptoBE->enero = (double)$row->enero;
            $oConceptoBE->febrero = (double)$row->febrero;
            $oConceptoBE->marzo = (double)$row->marzo;
            $oConceptoBE->abril = (double)$row->abril;
            $oConceptoBE->mayo = (double)$row->mayo;
            $oConceptoBE->junio = (double)$row->junio;
            $oConceptoBE->julio = (double)$row->julio;
            $oConceptoBE->agosto = (double)$row->agosto;
            $oConceptoBE->setiembre = (double)$row->setiembre;
            $oConceptoBE->octubre = (double)$row->octubre;
            $oConceptoBE->noviembre = (double)$row->noviembre;
            $oConceptoBE->diciembre = (double)$row->diciembre;
            $oConceptoBE->total = (double)$row->total;

            array_push($listResult, $oConceptoBE);
          }
          
          $retorno = 0;
          $mensaje = MSG_OK_GENERAL;
        }
        else
        {
          $retorno = 1;
          $mensaje = MSG_ERROR_GENERAL;
        }

        return $listResult;
    }
}
?>