<?php

class MaterialDAO extends CI_Model 
{
    public function __construct()
    {    
        parent::__construct();
    }

    public function getMaterial($oMaterialBE, &$retorno, &$mensaje)
    {
        $idMaterial = $oMaterialBE->idMaterial;
        $descripcion = $oMaterialBE->descripcion;

        $listResult = array();
              
        $sql = "SELECT
                    T1.idmaterial,
                    T1.idunidad,
                    T1.descripcion,
                    T1.observacion,
                    T1.precio,
                    (SELECT TT.descripcion FROM parametro TT WHERE TT.tipparam = 'UNIDAD_MEDIDA' AND TT.codigo = T1.idunidad AND TT.eliminado = 0 LIMIT 1) AS dscunidad
                FROM
                    material T1
                WHERE
                    COALESCE(?,0) IN (T1.idmaterial,0)
                    AND UPPER(CONCAT(T1.descripcion)) LIKE CONCAT('%',UPPER(COALESCE(?,'')),'%')
                    AND T1.eliminado = 0
                ORDER BY
                    3";
        
        $query = $this->db->query($sql, array($idMaterial, $descripcion));

        if($query)
        {
          $list = $query->result();

          foreach ($list as $row)
          {
            $oMaterialBE = new MaterialBE();

            $oMaterialBE->idMaterial = (int)$row->idmaterial;
            $oMaterialBE->idUnidad = $row->idunidad;
            $oMaterialBE->descripcion = $row->descripcion;
            $oMaterialBE->observacion = $row->observacion;
            $oMaterialBE->precio = (double)$row->precio;
            $oMaterialBE->dscUnidad = $row->dscunidad;

            array_push($listResult, $oMaterialBE);
          }
          
          $retorno = 0;
          $mensaje = MSG_OK_GENERAL;
        }
        else
        {
          $retorno = 1;
          $mensaje = MSG_ERROR_GENERAL;
        }

        return $listResult;
    }
    
    public function createMaterial($oMaterialBE, &$retorno, &$mensaje)
    {
        $this->load->model('UtilDAO', 'util');

        $objResult = null;
                
        $idUnidad = $oMaterialBE->idUnidad;
        $descripcion = $oMaterialBE->descripcion;
        $observacion = $oMaterialBE->observacion;
        $precio = $oMaterialBE->precio;
        $idUsuCrea = $oMaterialBE->idUsuCrea;

        $idMaterial = $this->util->getNextId("material","idmaterial");
                
        if ($idMaterial > 0)
        {
            $sql = "INSERT INTO material
                    (
                        idmaterial,
                        idunidad,
                        descripcion,
                        observacion,
                        precio,
                        idusucrea,
                        fecusucrea,
                        idusumod,
                        fecusumod,
                        eliminado
                    )
                    VALUES
                    (
                        ?,
                        ?,
                        ?,
                        ?,
                        ?,
                        ?,
                        NOW(),
                        NULL,
                        NULL,
                        0
                    )";
                
            $query = $this->db->query($sql, array($idMaterial,
                                                  $idUnidad,
                                                  $descripcion,
                                                  $observacion,
                                                  $precio,
                                                  $idUsuCrea));

            if($query)
            {
                $oMaterialBE = new MaterialBE();

                $oMaterialBE->idMaterial = $idMaterial;

                $listResult = $this->getMaterial($oMaterialBE, $retorno, $mensaje);

                if(count($listResult) > 0)
                {
                    $objResult = $listResult[0];

                    $retorno = 0;
                    $mensaje = "Material grabado correctamente";
                }
                else
                {
                    $retorno = 1;
                    $mensaje = MSG_ERROR_GENERAL;
                }
            }
            else
            {
                $retorno = 1;
                $mensaje = MSG_ERROR_GENERAL;
            }         
        }
        else
        {
            $retorno = 1;
            $mensaje = "Ocurrió un error al obtener el codigo de Material";
        }

        return $objResult;
    }

    public function updateMaterial($oMaterialBE, &$retorno, &$mensaje)
    {
        $this->load->model('UtilDAO', 'util');

        $objResult = null;                

        $idMaterial = $oMaterialBE->idMaterial;
        $idUnidad = $oMaterialBE->idUnidad;
        $descripcion = $oMaterialBE->descripcion;
        $observacion = $oMaterialBE->observacion;
        $precio = $oMaterialBE->precio;        
        $idUsuMod = $oMaterialBE->idUsuMod;        
        
        if ($idMaterial > 0)
        {
            $oMaterialBE = new MaterialBE();

            $oMaterialBE->idMaterial = $idMaterial;

            $listResult = $this->getMaterial($oMaterialBE, $retorno, $mensaje);

            if(count($listResult) > 0)
            {
                $sql = "UPDATE 
                            material
                        SET                        
                            idunidad = COALESCE(?,idunidad),
                            descripcion = COALESCE(?,descripcion),
                            observacion = COALESCE(?,observacion),
                            precio = COALESCE(?,precio),
                            idusumod = COALESCE(?,idusumod),
                            fecusumod = NOW()
                        WHERE
                            idmaterial = ?
                            AND eliminado = 0";
                    
                $query = $this->db->query($sql, array($idUnidad,
                                                      $descripcion,
                                                      $observacion,
                                                      $precio,
                                                      $idUsuMod,
                                                      $idMaterial));

                if($query)
                {
                    $oMaterialBE = new MaterialBE();

                    $oMaterialBE->idMaterial = $idMaterial;

                    $listResult = $this->getMaterial($oMaterialBE, $retorno, $mensaje);

                    if(count($listResult) > 0)
                    {
                        $objResult = $listResult[0];

                        $retorno = 0;
                        $mensaje = "Material grabado correctamente";
                    }
                    else
                    {
                        $retorno = 1;
                        $mensaje = MSG_ERROR_GENERAL;
                    }
                }
                else
                {
                    $retorno = 1;
                    $mensaje = MSG_ERROR_GENERAL;
                }
            }
            else
            {
                $retorno = 1;
                $mensaje = "El Material no existe";
            }
        }
        else
        {
            $retorno = 1;
            $mensaje = "Debe ingresar el codigo de Material";
        }

        return $objResult;
    }

    public function deleteMaterial($oMaterialBE, &$retorno, &$mensaje)
    {
        $this->load->model('UtilDAO', 'util');

        $objResult = null;                

        $idMaterial = $oMaterialBE->idMaterial;        
        $idUsuMod = $oMaterialBE->idUsuMod;
        
        if ($idMaterial > 0)
        {
            $oMaterialBE = new MaterialBE();

            $oMaterialBE->idMaterial = $idMaterial;

            $listResult = $this->getMaterial($oMaterialBE, $retorno, $mensaje);

            if(count($listResult) > 0)
            {                
                $sql = "UPDATE 
                           material
                        SET
                           idusumod = ?,
                           fecusumod = NOW(),
                           eliminado = 1
                        WHERE
                           idmaterial = ?
                           AND eliminado = 0";
                    
                $query = $this->db->query($sql, array($idUsuMod,
                                                      $idMaterial));
                if($query)
                {
                    $retorno = 0;
                    $mensaje = "Material eliminado correctamente";
                }
                else
                {
                    $retorno = 1;
                    $mensaje = MSG_ERROR_GENERAL;
                }
            }
            else
            {
                $retorno = 1;
                $mensaje = "El Material no existe";
            }                    
        }
        else
        {
            $retorno = 1;
            $mensaje = "Debe ingresar el codigo de Material";
        }

        return $objResult;
    }    
}
?>